@@ -16,7 +16,7 @@
 TODO:   - expand this for use with type_info() 
 	- map all types to closest sql type.
 	- set up quote functions for remaining types
-	- autogeneratet this file.
+	- autogenerate this file.
 */
 
 static sql_type_info_t pg_types[] = {
@@ -57,8 +57,8 @@ static sql_type_info_t pg_types[] = {
 	{VARCHAROID, "varchar", DBDPG_TRUE, quote_varchar, dequote_varchar, {SQL_VARCHAR}},
 	{DATEOID, "date", DBDPG_TRUE, null_quote, null_dequote, {0}},
 	{TIMEOID, "time", DBDPG_TRUE, null_quote, null_dequote, {0}},
-	{TIMESTAMPOID, "timestamp", DBDPG_TRUE, null_quote, null_dequote, {0}},
-	{TIMESTAMPTZOID, "datetime", DBDPG_TRUE, null_quote, null_dequote, {0}},
+	{TIMESTAMPOID, "timestamp", DBDPG_TRUE, null_quote, null_dequote, {SQL_TYPE_TIMESTAMP}},
+	{TIMESTAMPTZOID, "datetime", DBDPG_TRUE, null_quote, null_dequote, {SQL_TYPE_TIMESTAMP_WITH_TIMEZONE}},
 	{INTERVALOID, "timespan", DBDPG_TRUE, null_quote, null_dequote, {0}},
 	{TIMETZOID, "timestamptz", DBDPG_TRUE, null_quote, null_dequote, {0}},
 	{BITOID, "bitstring", DBDPG_TRUE, null_quote, null_dequote, {0}},
